<?php
/*
* Gallery - a web based photo album viewer and editor
* Copyright (C) 2000-2008 Bharat Mediratta
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.
*
* $Id: album.php 17840 2008-08-12 01:08:31Z JensT $
*/

/**
 * @package	libs
 * @author	Jens Tkotz
 */


/**
 * A class to parse color values.
 * Ported to PHP by Jens Tkotz
 * @author Stoyan Stefanov <sstoo@gmail.com>
 * @link   http://www.phpied.com/rgb-color-parser-in-javascript/
 * @license Use it if you like it
 */
class RGBColor {
	var $ok;
	var $r;
	var $g;
	var $b;
	var $hex;
	var $brightness;

	function RGBColor($color_string) {

		$this->ok = false;

		// strip any leading #
		if(substr($color_string, 0,1) == '#') {
			$color_string = substr($color_string, 1);
		}

		// replace all spaces
		$color_string = str_replace('', ' ', $color_string);

		$color_string = strtolower($color_string);

		// before getting into regexps, try simple matches and overwrite the input
		$simple_colors = array(
			'aliceblue' => 'f0f8ff',
			'antiquewhite' => 'faebd7',
			'aqua' => '00ffff',
			'aquamarine' => '7fffd4',
			'azure' => 'f0ffff',
			'beige' => 'f5f5dc',
			'bisque' => 'ffe4c4',
			'black' => '000000',
			'blanchedalmond' => 'ffebcd',
			'blue' => '0000ff',
			'blueviolet' => '8a2be2',
			'brown' => 'a52a2a',
			'burlywood' => 'deb887',
			'cadetblue' => '5f9ea0',
			'chartreuse' => '7fff00',
			'chocolate' => 'd2691e',
			'coral' => 'ff7f50',
			'cornflowerblue' => '6495ed',
			'cornsilk' => 'fff8dc',
			'crimson' => 'dc143c',
			'cyan' => '00ffff',
			'darkblue' => '00008b',
			'darkcyan' => '008b8b',
			'darkgoldenrod' => 'b8860b',
			'darkgray' => 'a9a9a9',
			'darkgreen' => '006400',
			'darkkhaki' => 'bdb76b',
			'darkmagenta' => '8b008b',
			'darkolivegreen' => '556b2f',
			'darkorange' => 'ff8c00',
			'darkorchid' => '9932cc',
			'darkred' => '8b0000',
			'darksalmon' => 'e9967a',
			'darkseagreen' => '8fbc8f',
			'darkslateblue' => '483d8b',
			'darkslategray' => '2f4f4f',
			'darkturquoise' => '00ced1',
			'darkviolet' => '9400d3',
			'deeppink' => 'ff1493',
			'deepskyblue' => '00bfff',
			'dimgray' => '696969',
			'dodgerblue' => '1e90ff',
			'feldspar' => 'd19275',
			'firebrick' => 'b22222',
			'floralwhite' => 'fffaf0',
			'forestgreen' => '228b22',
			'fuchsia' => 'ff00ff',
			'gainsboro' => 'dcdcdc',
			'ghostwhite' => 'f8f8ff',
			'gold' => 'ffd700',
			'goldenrod' => 'daa520',
			'gray' => '808080',
			'green' => '008000',
			'greenyellow' => 'adff2f',
			'honeydew' => 'f0fff0',
			'hotpink' => 'ff69b4',
			'indianred ' => 'cd5c5c',
			'indigo ' => '4b0082',
			'ivory' => 'fffff0',
			'khaki' => 'f0e68c',
			'lavender' => 'e6e6fa',
			'lavenderblush' => 'fff0f5',
			'lawngreen' => '7cfc00',
			'lemonchiffon' => 'fffacd',
			'lightblue' => 'add8e6',
			'lightcoral' => 'f08080',
			'lightcyan' => 'e0ffff',
			'lightgoldenrodyellow' => 'fafad2',
			'lightgrey' => 'd3d3d3',
			'lightgreen' => '90ee90',
			'lightpink' => 'ffb6c1',
			'lightsalmon' => 'ffa07a',
			'lightseagreen' => '20b2aa',
			'lightskyblue' => '87cefa',
			'lightslateblue' => '8470ff',
			'lightslategray' => '778899',
			'lightsteelblue' => 'b0c4de',
			'lightyellow' => 'ffffe0',
			'lime' => '00ff00',
			'limegreen' => '32cd32',
			'linen' => 'faf0e6',
			'magenta' => 'ff00ff',
			'maroon' => '800000',
			'mediumaquamarine' => '66cdaa',
			'mediumblue' => '0000cd',
			'mediumorchid' => 'ba55d3',
			'mediumpurple' => '9370d8',
			'mediumseagreen' => '3cb371',
			'mediumslateblue' => '7b68ee',
			'mediumspringgreen' => '00fa9a',
			'mediumturquoise' => '48d1cc',
			'mediumvioletred' => 'c71585',
			'midnightblue' => '191970',
			'mintcream' => 'f5fffa',
			'mistyrose' => 'ffe4e1',
			'moccasin' => 'ffe4b5',
			'navajowhite' => 'ffdead',
			'navy' => '000080',
			'oldlace' => 'fdf5e6',
			'olive' => '808000',
			'olivedrab' => '6b8e23',
			'orange' => 'ffa500',
			'orangered' => 'ff4500',
			'orchid' => 'da70d6',
			'palegoldenrod' => 'eee8aa',
			'palegreen' => '98fb98',
			'paleturquoise' => 'afeeee',
			'palevioletred' => 'd87093',
			'papayawhip' => 'ffefd5',
			'peachpuff' => 'ffdab9',
			'peru' => 'cd853f',
			'pink' => 'ffc0cb',
			'plum' => 'dda0dd',
			'powderblue' => 'b0e0e6',
			'purple' => '800080',
			'red' => 'ff0000',
			'rosybrown' => 'bc8f8f',
			'royalblue' => '4169e1',
			'saddlebrown' => '8b4513',
			'salmon' => 'fa8072',
			'sandybrown' => 'f4a460',
			'seagreen' => '2e8b57',
			'seashell' => 'fff5ee',
			'sienna' => 'a0522d',
			'silver' => 'c0c0c0',
			'skyblue' => '87ceeb',
			'slateblue' => '6a5acd',
			'slategray' => '708090',
			'snow' => 'fffafa',
			'springgreen' => '00ff7f',
			'steelblue' => '4682b4',
			'tan' => 'd2b48c',
			'teal' => '008080',
			'thistle' => 'd8bfd8',
			'tomato' => 'ff6347',
			'turquoise' => '40e0d0',
			'violet' => 'ee82ee',
			'violetred' => 'd02090',
			'wheat' => 'f5deb3',
			'white' => 'ffffff',
			'whitesmoke' => 'f5f5f5',
			'yellow' => 'ffff00',
			'yellowgreen' => '9acd32'
		);


		foreach ($simple_colors as $color_code) {
			if ($color_string == $color_code) {
				$color_string = $simple_colors[$color_code];
			}
		}
		// emd of simple type-in colors

		// array of color definition objects
		$color_defs = array(
			array(
				're' 		=> '/^rgb\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3})\)$/',
				'example'	=> "['rgb(123, 234, 45)', 'rgb(255,234,245)']",
				'base'		=> 10
			),
			array(
				're'		=> '/^(\w{2})(\w{2})(\w{2})$/',
				'example'	=> "['#00ff00', '336699']",
				'base'		=> 16
			),
			array(
				're'		=> '/^(\w{1})(\w{1})(\w{1})$/',
				'example'	=> "['#fb0', 'f0f']",
				'base'		=> 16
			)
		);



		// search through the definitions to find a match
		foreach($color_defs as $color_def) {
			if(! $this->ok && preg_match($color_def['re'], $color_string, $matches)) {
				$this->r = intval($matches[1], $color_def['base']);
				$this->g = intval($matches[2], $color_def['base']);
				$this->b = intval($matches[3], $color_def['base']);
				$this->ok = true;
			}
		}

		// validate/cleanup values
		$this->r = (!isset($this->r)) ? 0 : (($this->r > 255) ? 255 : $this->r);
		$this->g = (!isset($this->g)) ? 0 : (($this->g > 255) ? 255 : $this->g);
		$this->b = (!isset($this->b)) ? 0 : (($this->r > 255) ? 255 : $this->b);

		$this->calcHex();

		$this->calcBrightness();
	}

	function getRGBarray() {
		return array($this->r, $this->g, $this->b);
	}

	function getHex($withSharp = true) {
		if($withSharp) {
			return '#' . $this->hex;
		}
		else {
			return $this->hex;
		}
	}

	// < 128 ? 'white' : 'black';
	function getBrightness() {
		if($this->brightness > 128) {
			return true;
		}
		else {
			return false;
		}
	}

	function calcHex() {
		$r = dechex($this->r);
		$g = dechex($this->g);
		$b = dechex($this->b);

		$r = str_pad($r, 2, '0', STR_PAD_LEFT);
		$g = str_pad($g, 2, '0', STR_PAD_LEFT);
		$b = str_pad($b, 2, '0', STR_PAD_LEFT);

		$this->hex = "$r$g$b";
	}

	/* Credits for this function goes to the horde project.
	* http://horde.org
	*/
	function calcBrightness() {
		$r = hexdec(substr($this->hex, 0, 2));
		$g = hexdec(substr($this->hex, 2, 2));
		$b = hexdec(substr($this->hex, 4, 2));

		$this->brightness = (($r * 299) + ($g * 587) + ($b * 114)) / 1000;
	}

	function getDarken($withSharp = true) {
		$r = $this->r - 30;
		$g = $this->g - 30;
		$b = $this->b - 30;

		if($r < 0) $r = 0;
		if($g < 0) $g = 0;
		if($b < 0) $b = 0;

		$r = str_pad(dechex($r), 2, '0', STR_PAD_LEFT);
		$g = str_pad(dechex($g), 2, '0', STR_PAD_LEFT);
		$b = str_pad(dechex($b), 2, '0', STR_PAD_LEFT);

		if($withSharp) {
			return "#$r$g$b";
		}
		else {
			return "$r$g$b";
		}
	}

	function getLighten($withSharp = true) {
		$r = $this->r + 30;
		$g = $this->g + 30;
		$b = $this->b + 30;

		if($r > 255) $r = 255;
		if($g > 255) $g = 255;
		if($b > 255) $b = 255;

		$r = str_pad(dechex($r), 2, '0', STR_PAD_LEFT);
		$g = str_pad(dechex($g), 2, '0', STR_PAD_LEFT);
		$b = str_pad(dechex($b), 2, '0', STR_PAD_LEFT);

		if($withSharp) {
			return "#$r$g$b";
		}
		else {
			return "$r$g$b";
		}
	}
}